@@ -4,9 +4,9 @@ use Carp;
 use DBI;
 use strict;
 use Class::ReturnValue;
-use vars qw($VERSION @ISA $DBIHandle $DEBUG $TRANSCOUNT);
+use vars qw($VERSION @ISA $DBIHandle $DEBUG $TRANSDEPTH);
 
-$TRANSCOUNT = 0;
+$TRANSDEPTH = 0;
 
 $VERSION = '$Version$';
 
@@ -117,6 +117,10 @@ sub Connect  {
   $self->BuildDSN(%args);
 
   my $handle = DBI->connect($self->DSN, $args{'User'}, $args{'Password'}) || croak "Connect Failed $DBI::errstr\n" ;
+ 
+  #databases do case conversion on the name of columns returned. 
+  #actually, some databases just ignore case. this smashes it to something consistent 
+  $handle->{FetchHashKeyName} ='NAME_lc';
 
   #Set the handle 
   $self->dbh($handle);
@@ -127,7 +131,7 @@ sub Connect  {
 
 # {{{ BuildDSN
 
-=item  BuildDSN PARAMHASH
+=head2 BuildDSN PARAMHASH
 
 Takes a bunch of parameters:  
 
@@ -162,7 +166,7 @@ sub BuildDSN {
 
 # {{{ DSN
 
-=item DSN
+=head2 DSN
 
     Returns the DSN for this database connection.
 
@@ -356,6 +360,7 @@ sub SimpleQuery  {
     my $self = shift;
     my $QueryString = shift;
     my @bind_values = (@_);
+
  
     my $sth = $self->dbh->prepare($QueryString);
     unless ($sth) {
@@ -373,6 +378,15 @@ sub SimpleQuery  {
 	    return ($ret->return_value);
 	}
     }
+    # Check @bind_values for HASH refs 
+    for (my $bind_idx = 0; $bind_idx < scalar @bind_values; $bind_idx++) {
+        if (ref($bind_values[$bind_idx]) eq "HASH") {
+            my $bhash = $bind_values[$bind_idx];
+            $bind_values[$bind_idx] = $bhash->{'value'};
+            delete $bhash->{'value'};
+            $sth->bind_param($bind_idx+1, undef, $bhash );
+        }
+    }
     unless ( $sth->execute(@bind_values) ) {
         if ($DEBUG) {
             die "$self couldn't execute the query '$QueryString'"
@@ -440,6 +454,41 @@ sub BinarySafeBLOBs {
 
 # }}}
 
+# {{{ KnowsBLOBs
+
+=head2 KnowsBLOBs
+
+Returns 1 if the current database supports inserts of BLOBs automatically.
+Returns undef if the current database must be informed of BLOBs for inserts.
+
+=cut
+
+sub KnowsBLOBs {
+    my $self = shift;
+    return(1);
+}
+
+# }}}
+
+# {{{ BLOBParams
+
+=head2 BLOBParams FIELD_NAME FIELD_TYPE
+
+Returns a hash ref for the bind_param call to identify BLOB types used by 
+the current database for a particular column type.                 
+
+=cut
+
+sub BLOBParams {
+    my $self = shift;
+    # Don't assign to key 'value' as it is defined later. 
+    return ( {} );
+}
+
+# }}}
+
+# {{{ DatabaseVersion
+
 =head2 DatabaseVersion
 
 Returns the database's version. The base implementation uses a "SELECT VERSION"
@@ -456,7 +505,7 @@ sub DatabaseVersion {
         $self->{'database_version'}= $vals[0];
     }
 }
-
+# }}}
 
 # {{{ CaseSensitive
 
@@ -490,11 +539,11 @@ Emulates nested transactions, by keeping a transaction stack depth.
 
 sub BeginTransaction {
     my $self = shift;
-    $TRANSCOUNT++;
-    if ($TRANSCOUNT > 1 ) {
-        return ($TRANSCOUNT);
+    $TRANSDEPTH++;
+    if ($TRANSDEPTH > 1 ) {
+        return ($TRANSDEPTH);
     } else {
-        return($self->dbh->begin_work);
+       return($self->dbh->begin_work);
     }
 }
 
@@ -511,13 +560,13 @@ This will turn Autocommit mode back on.
 
 sub Commit {
     my $self = shift;
-    unless ($TRANSCOUNT) {Carp::confess("Attempted to commit a transaction with none in progress")};
-    $TRANSCOUNT--;
+    unless ($TRANSDEPTH) {Carp::confess("Attempted to commit a transaction with none in progress")};
+    $TRANSDEPTH--;
 
-    if ($TRANSCOUNT == 0 ) {
+    if ($TRANSDEPTH == 0 ) {
         return($self->dbh->commit);
     } else { #we're inside a transaction
-        return($TRANSCOUNT);
+        return($TRANSDEPTH);
     }
 }
 
@@ -525,27 +574,61 @@ sub Commit {
 
 # {{{ Rollback
 
-=head2 Rollback
+=head2 Rollback [FORCE]
 
 Tells DBIx::SearchBuilder to abort the current SQL transaction. 
 This will turn Autocommit mode back on.
 
+If this method is passed a true argument, stack depth is blown away and the outermost transaction is rolled back
+
 =cut
 
 sub Rollback {
     my $self = shift;
-    unless ($TRANSCOUNT) {Carp::confess("Attempted to rollback a transaction with none in progress")};
-    $TRANSCOUNT--;
+    my $force = shift || undef;
+    #unless ($TRANSDEPTH) {Carp::confess("Attempted to rollback a transaction with none in progress")};
+    $TRANSDEPTH--;
 
-    if ($TRANSCOUNT == 0 ) {
-        return($self->dbh->rollback);
+    if ($force) {
+        $TRANSDEPTH = 0;
+       return($self->dbh->rollback);
+    }
+
+    if ($TRANSDEPTH == 0 ) {
+       return($self->dbh->rollback);
     } else { #we're inside a transaction
-        return($TRANSCOUNT);
+        return($TRANSDEPTH);
     }
 }
 
 # }}}
 
+=head2 ForceRollback
+
+Force the handle to rollback. Whether or not we're deep in nested transactions
+
+=cut
+
+sub ForceRollback {
+    my $self = shift;
+    $self->Rollback(1);
+}
+
+
+=head2 TransactionDepth
+
+Return the current depth of the faked nested transaction stack.
+
+=cut
+
+sub TransactionDepth {
+    my $self = shift;
+    return ($TRANSDEPTH); 
+}
+
+
+# {{{ ApplyLimits
+
 =head2 ApplyLimits STATEMENTREF ROWS_PER_PAGE FIRST_ROW
 
 takes an SQL SELECT statement and massages it to return ROWS_PER_PAGE starting with FIRST_ROW;
@@ -574,6 +657,94 @@ sub ApplyLimits {
 }
 
 
+# }}}
+
+
+# {{{ Join
+
+=head2 Join { Paramhash }
+
+Takes a paramhash of everything Searchbuildler::Record does + a parameter called 'SearchBuilder that contains a ref to a SearchBuilder object'.
+This performs the join.
+
+
+=cut
+
+
+sub Join {
+
+    my $self = shift;
+    my %args = (
+                SearchBuilder => undef,
+                 TYPE   => 'normal',
+                 FIELD1 => undef, ALIAS1 => undef,
+                 TABLE2 => undef, FIELD2 => undef, ALIAS2 => undef,
+                 @_ );
+
+
+
+    if ( $args{'TYPE'} =~ /LEFT/i ) {
+        my $alias = $args{'SearchBuilder'}->_GetAlias( $args{'TABLE2'} );
+
+        $args{'SearchBuilder'}->{'left_joins'}{"$alias"}{'alias_string'} =
+          " LEFT JOIN $args{'TABLE2'} as $alias ";
+
+        $args{'SearchBuilder'}->{'left_joins'}{"$alias"}{'criteria'}{'base_criterion'} =
+          " $args{'ALIAS1'}.$args{'FIELD1'} = $alias.$args{'FIELD2'}";
+
+        return ($alias);
+    }
+
+    # we need to build the table of links.
+    my $clause =
+      $args{'ALIAS1'} . "."
+      . $args{'FIELD1'} . " = "
+      . $args{'ALIAS2'} . "."
+      . $args{'FIELD2'};
+    $args{'SearchBuilder'}->{'table_links'} .= " AND $clause ";
+
+}
+
+# }}}
+
+# {{{ DistinctQuery
+
+=head2 DistinctQuery STATEMENTREF 
+
+takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result set.
+
+
+=cut
+
+sub DistinctQuery {
+    my $self = shift;
+    my $statementref = shift;
+    #my $table = shift;
+
+    # Prepend select query for DBs which allow DISTINCT on all column types.
+    $$statementref = "SELECT DISTINCT main.* FROM $$statementref";
+
+}
+
+# }}}
+
+
+
+=head2 DESTROY
+
+When we get rid of the Searchbuilder::Handle, we need to disconnect from the database
+
+=cut
+
+  
+sub DESTROY {
+  my $self = shift;
+  if( defined $self->dbh ) {
+    $self->Disconnect();
+  }
+}
+
+
 1;
 __END__
 
